/* $Id: pcadump.c,v 1.5 1997/04/17 14:47:12 ud3 Exp $ */

#include <linux/atm.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "pca200e.h"

#define DUMP_DEVICE	0x00000001
#define DUMP_CPQUEUE	0x00000002
#define DUMP_MONITOR	0x00000004
#define DUMP_STATS	0x00000008

/* prototypes */
int usage(char*, int);
unsigned int dmaswap(unsigned int in);
void showdump(struct pca200e_dump *, int, unsigned long);

int
usage(char* name, int rv)
{
  printf("shows state and statistics of PCA-200EPC\n\n"
	 "usage: %s [-d] [-c] [-m] [-s] [-a] [-h] <itf> [<itf> ...]\n"
	 "  <itf>\t\tatm interface number\n"
	 "  -d\t\tshow device info (default)\n"
	 "  -c\t\tshow cpqueue info\n"
	 "  -m\t\tshow monitor info\n"
	 "  -s\t\tshow device statistics\n"
	 "  -a\t\tlike -d -c -m -s\n"
	 "  -h\t\tshow this help text\n\n",
	 name);
  exit(rv);
};

unsigned int
dmaswap(unsigned int in)
{
  asm("bswap %%eax\n"
      : "=a" (in)
      : "a" (in));
  return in;
};

void
showdump(struct pca200e_dump* dump, int itf, unsigned long flags)
{
  printf("FORE Systems PCA-200E dump for itf %d\n", itf);
  if (flags & DUMP_DEVICE)
    {
      printf("\n device:\n");
      printf("  internal name:\t%s\n", dump->dev.name);
      printf("  PCI bus number:\t%d\n", dump->dev.bus);
      printf("  device function:\t0x%x\n", dump->dev.dev_fn);
      printf("  interrupt line:\t%d\n", dump->dev.irq);
      printf("  physical rambase:\t0x%08x\n", (unsigned) dump->dev.phys_rambase);
      printf("  virtual rambase:\t0x%08x\n", (unsigned) dump->dev.virt_rambase);
      printf("  HW address (ESI):\t%02x:%02x:%02x:%02x:%02x:%02x\n",
	     dump->dev.mac[0], dump->dev.mac[1], dump->dev.mac[2],
	     dump->dev.mac[3], dump->dev.mac[4], dump->dev.mac[5]);
      printf("  active small buffs:\t%d\n", dump->dev.smlb1_count);
      printf("  active large buffs:\t%d\n", dump->dev.lrgb1_count);
      printf("  card init state:\t%d\n", dump->dev.cardstate);


    };
  if (flags & DUMP_CPQUEUE)
    {
      printf("\n cpqueue:\n");
      printf("  heartbeat state:\t0x%08X\n", dump->cpqueue.heartbeat);
      printf("  firmware release:\t%d.%d.%d\n",
	     dump->cpqueue.firmware_release >> 16,
	     dump->cpqueue.firmware_release << 16 >> 24,
	     dump->cpqueue.firmware_release << 24 >> 24);
      printf("  monitor release:\t%d.%d\n",
	     dump->cpqueue.mon960_release >> 16,
	     dump->cpqueue.mon960_release << 16 >> 16);
      printf("  media type id:\t%d\n", dump->cpqueue.media_type);
      // printf("  OC3 revision: \t%d\n", dump->cpqueue.oc3_revision);
    };
  if (flags & DUMP_MONITOR)
    {
      printf("\n monitor:\n");
      printf("  boot status word:\t0x%08x\n", dump->monitor.bstat);
      printf("  version number:\t%d\n", dump->monitor.version);
    };
  if (flags & DUMP_STATS)
    {
      stats200_t* stat = &dump->stats;
      printf("\n device statistics:\n");
      printf("  PHY_4b5b:\n"
	     "   crc_header_errors: %d, framing_errors: %d\n",
	     dmaswap(stat->phy4b5b.crc_header_errors),
	     dmaswap(stat->phy4b5b.framing_errors));
      
      printf("  PHY_OC3:\n"
	     "   section_bip8_errors: %d,\tpath_bip8_errors: %d,\tline_bip24_errors: %d\n"
             "   line_febe_errors:    %d,\tpath_febe_errors: %d\n"
	     "   corr_hcs_errors:     %d,\tucorr_hcs_errors: %d\n",
	     dmaswap(stat->phyoc3.section_bip8_errors),
	     dmaswap(stat->phyoc3.path_bip8_errors),
	     dmaswap(stat->phyoc3.line_bip24_errors),
	     dmaswap(stat->phyoc3.line_febe_errors),
	     dmaswap(stat->phyoc3.path_febe_errors),
	     dmaswap(stat->phyoc3.corr_hcs_errors),
	     dmaswap(stat->phyoc3.ucorr_hcs_errors));

      printf("  ATM:\n"
	     "   TX: %d, RX: %d\n",
	     dmaswap(stat->atm.cells_transmitted),
	     dmaswap(stat->atm.cells_received));
      printf("   vpi out of range: %d, vpi no conn: %d\n",
	     dmaswap(stat->atm.vpi_bad_range),
	     dmaswap(stat->atm.vpi_no_conn));
      printf("   vci out of range: %d, vci no conn: %d\n",
	     dmaswap(stat->atm.vci_bad_range),
	     dmaswap(stat->atm.vci_no_conn));
      
      printf("  AAL0:\n"
	     "   TX: %d, RX: %d, dropped: %d\n",
	     dmaswap(stat->aal0.cells_transmitted),
	     dmaswap(stat->aal0.cells_received),
	     dmaswap(stat->aal0.cells_dropped));
      
      printf("  AAL4:\n"
	     "   SAR sublayer: TX: %d, RX: %d, dropped: %d, CRC errors: %d, protocol errors: %d\n"
	     "   CS  sublayer: TX: %d, RX: %d, dropped: %d,                protocol errors: %d\n",
	     dmaswap(stat->aal4.cells_transmitted),
	     dmaswap(stat->aal4.cells_received),
	     dmaswap(stat->aal4.cells_dropped),
	     dmaswap(stat->aal4.cells_crc_errors),
	     dmaswap(stat->aal4.cells_protocol_errors),
	     dmaswap(stat->aal4.cspdus_transmitted),
	     dmaswap(stat->aal4.cspdus_received),
	     dmaswap(stat->aal4.cspdus_dropped),
	     dmaswap(stat->aal4.cspdus_protocol_errors));

      printf("  AAL5:\n"
	     "   SAR sublayer: TX: %d, RX: %d, dropped: %d, congestions: %d\n"
	     "   CS  sublayer: TX: %d, RX: %d, dropped: %d, CRC errors:  %d, protocol errors: %d\n",
	     dmaswap(stat->aal5.cells_transmitted),
	     dmaswap(stat->aal5.cells_received),
	     dmaswap(stat->aal5.cells_dropped),
	     dmaswap(stat->aal5.congestion_experienced),
	     dmaswap(stat->aal5.cspdus_transmitted),
	     dmaswap(stat->aal5.cspdus_received),
	     dmaswap(stat->aal5.cspdus_dropped),
	     dmaswap(stat->aal5.cspdus_crc_errors),
	     dmaswap(stat->aal5.cspdus_protocol_errors));

      printf("  AUX:\n"
	     "   allocation failures:\n"
	     "    small b1:\t%d, large b1: %d\n"
	     "    small b2:\t%d, large b2: %d\n"
	     "    RX PDUs:\t%d\n",
	     dmaswap(stat->aux200.small_b1_failed),
	     dmaswap(stat->aux200.large_b1_failed),
	     dmaswap(stat->aux200.small_b2_failed),
	     dmaswap(stat->aux200.large_b2_failed),
	     dmaswap(stat->aux200.rpd_alloc_failed));
      printf("  receive carrier is\t%s\n",stat->aux200.receive_carrier?"ON":"OFF!");
  

    };
};


int
main(int argc, char* argv[])
{
  int s;
  int rc;
  unsigned long flags; 
  struct atmif_sioc req;
  struct pca200e_dump dump;
  
  flags = DUMP_DEVICE;

  optind = 0;
  while ((rc = getopt(argc,argv,"dcmsah")) != EOF)
    switch (rc) {
    case 'd':
      flags |= DUMP_DEVICE;
      break;
    case 'c':
      flags |= DUMP_CPQUEUE;
      break;
    case 'm':
      flags |= DUMP_MONITOR;
      break;
    case 's':
      flags |= DUMP_STATS;
      break;
    case 'a':
      flags |= ~0;
      break;
    case 'h':
      usage(argv[0], 0);
    default :
      usage(argv[0], -1);
    };
  
  if (!(optind < argc))
      usage(argv[0], -1);

  while (optind < argc)
    {
      memset(&dump, 0, sizeof(struct pca200e_dump));
      req.number = atol(argv[optind++]);
      req.arg = &dump;
      req.length = sizeof(dump);
      s = socket(PF_ATMPVC,SOCK_DGRAM,0);
      if ((rc = ioctl(s, PCA200E_DUMP, &req)) < 0)
	printf("\n%s: no interface %d\n", argv[0], req.number);
      else
	showdump(&dump, req.number, flags);
    };
  
  return 0;
};


